@@ -2,6 +2,7 @@ import asyncio
 import os
 import re
 import subprocess
+import weakref
 from typing import Any, AsyncIterator, Callable, Iterator, List, Optional, Union
 
 
@@ -21,17 +22,28 @@ class FfmpegProgress:
     TIME_REGEX = re.compile(
         r"out_time=(?P<hour>\d{2}):(?P<min>\d{2}):(?P<sec>\d{2})\.(?P<ms>\d{2})"
     )
+    PROGRESS_REGEX = re.compile(r"[a-z0-9_]+=.+")
 
-    def __init__(self, cmd: List[str], dry_run: bool = False) -> None:
+    def __init__(
+        self,
+        cmd: List[str],
+        dry_run: bool = False,
+        exclude_progress: bool = False,
+        ffprobe_path: str = "ffprobe",
+    ) -> None:
         """Initialize the FfmpegProgress class.
 
         Args:
             cmd (List[str]): A list of command line elements, e.g. ["ffmpeg", "-i", ...]
             dry_run (bool, optional): Only show what would be done. Defaults to False.
+            exclude_progress (bool, optional): Exclude progress lines from output. Defaults to False.
+            ffprobe_path (str, optional): Path to ffprobe executable. Defaults to "ffprobe".
         """
         self.cmd = cmd
         self.stderr: Union[str, None] = None
         self.dry_run = dry_run
+        self.exclude_progress = exclude_progress
+        self.ffprobe_path = ffprobe_path
         self.process: Any = None
         self.stderr_callback: Union[Callable[[str], None], None] = None
         self.base_popen_kwargs = {
@@ -48,8 +60,61 @@ class FfmpegProgress:
 
         self.current_input_idx: int = 0
         self.total_dur: Union[None, int] = None
-        if FfmpegProgress._uses_error_loglevel(self.cmd):
-            self.total_dur = FfmpegProgress._probe_duration(self.cmd)
+        # Skip probing duration in dry-run mode to avoid running ffprobe
+        if not self.dry_run and FfmpegProgress._uses_error_loglevel(self.cmd):
+            self.total_dur = self._probe_duration(self.cmd)
+
+        # Set up cleanup on garbage collection as a fallback
+        self._cleanup_ref = weakref.finalize(self, self._cleanup_process, None)
+
+    @staticmethod
+    def _cleanup_process(process: Any) -> None:
+        """Clean up a process if it's still running."""
+        if process is not None and hasattr(process, "poll"):
+            try:
+                if process.poll() is None:  # Process is still running
+                    process.kill()
+                    if hasattr(process, "wait"):
+                        try:
+                            process.wait(timeout=1.0)
+                        except subprocess.TimeoutExpired:
+                            pass  # Process didn't terminate gracefully, but we killed it
+            except Exception:
+                pass  # Ignore any errors during cleanup
+
+    def __del__(self) -> None:
+        """Fallback cleanup when object is garbage collected."""
+        if hasattr(self, "process") and self.process is not None:
+            self._cleanup_process(self.process)
+
+    def __enter__(self) -> "FfmpegProgress":
+        """Context manager entry."""
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
+        """Context manager exit - ensures process cleanup."""
+        if self.process is not None:
+            try:
+                if hasattr(self.process, "poll") and self.process.poll() is None:
+                    self.quit()
+            except Exception:
+                pass  # Ignore errors during cleanup
+
+    async def __aenter__(self) -> "FfmpegProgress":
+        """Async context manager entry."""
+        return self
+
+    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
+        """Async context manager exit - ensures process cleanup."""
+        if self.process is not None:
+            try:
+                if (
+                    hasattr(self.process, "returncode")
+                    and self.process.returncode is None
+                ):
+                    await self.async_quit()
+            except Exception:
+                pass  # Ignore errors during cleanup
 
     def _process_output(
         self,
@@ -73,7 +138,14 @@ class FfmpegProgress:
             self.stderr_callback(stderr_line)
 
         stderr.append(stderr_line.strip())
-        self.stderr = "\n".join(stderr)
+        self.stderr = "\n".join(
+            filter(
+                lambda line: not (
+                    self.exclude_progress and self.PROGRESS_REGEX.match(line)
+                ),
+                stderr,
+            )
+        )
 
         progress: Union[float, None] = None
         # assign the total duration if it was found. this can happen multiple times for multiple inputs,
@@ -109,8 +181,7 @@ class FfmpegProgress:
 
         return progress
 
-    @staticmethod
-    def _probe_duration(cmd: List[str]) -> Optional[int]:
+    def _probe_duration(self, cmd: List[str]) -> Optional[int]:
         """
         Get the duration via ffprobe from input media file
         in case ffmpeg was run with loglevel=error.
@@ -139,7 +210,7 @@ class FfmpegProgress:
             try:
                 output = subprocess.check_output(
                     [
-                        "ffprobe",
+                        self.ffprobe_path,
                         "-loglevel",
                         "error",
                         "-hide_banner",
@@ -236,29 +307,52 @@ class FfmpegProgress:
 
         self.process = subprocess.Popen(self.cmd_with_progress, **base_popen_kwargs)  # type: ignore
 
-        yield 0
+        # Update the cleanup finalizer with the actual process
+        self._cleanup_ref.detach()
+        self._cleanup_ref = weakref.finalize(self, self._cleanup_process, self.process)
 
-        stderr: List[str] = []
-        while True:
-            if self.process.stdout is None:
-                continue
+        try:
+            yield 0
 
-            stderr_line: str = (
-                self.process.stdout.readline().decode("utf-8", errors="replace").strip()
-            )
+            stderr: List[str] = []
+            while True:
+                if self.process.stdout is None:
+                    continue
 
-            if stderr_line == "" and self.process.poll() is not None:
-                break
+                stderr_line: str = (
+                    self.process.stdout.readline()
+                    .decode("utf-8", errors="replace")
+                    .strip()
+                )
 
-            progress = self._process_output(stderr_line, stderr, duration_override)
-            if progress is not None:
-                yield progress
+                if stderr_line == "" and self.process.poll() is not None:
+                    break
 
-        if self.process.returncode != 0:
-            raise RuntimeError(f"Error running command {self.cmd}: {self.stderr}")
+                progress = self._process_output(stderr_line, stderr, duration_override)
+                if progress is not None:
+                    yield progress
 
-        yield 100
-        self.process = None
+            if self.process.returncode != 0:
+                raise RuntimeError(f"Error running command {self.cmd}: {self.stderr}")
+
+            yield 100
+        finally:
+            # Ensure process cleanup even if an exception occurs
+            if self.process is not None:
+                try:
+                    if self.process.poll() is None:  # Process is still running
+                        self.process.kill()
+                        try:
+                            self.process.wait(timeout=1.0)
+                        except subprocess.TimeoutExpired:
+                            pass  # Process didn't terminate gracefully, but we killed it
+                except Exception:
+                    pass  # Ignore any errors during cleanup
+                finally:
+                    self.process = None
+                    # Detach the finalizer since we've cleaned up manually
+                    if hasattr(self, "_cleanup_ref"):
+                        self._cleanup_ref.detach()
 
     async def async_run_command_with_progress(
         self, popen_kwargs=None, duration_override: Union[float, None] = None
@@ -298,30 +392,65 @@ class FfmpegProgress:
             **base_popen_kwargs,  # type: ignore
         )
 
-        yield 0
+        # Update the cleanup finalizer with the actual process
+        self._cleanup_ref.detach()
+        self._cleanup_ref = weakref.finalize(self, self._cleanup_process, self.process)
 
-        stderr: List[str] = []
-        while True:
-            if self.process.stdout is None:
-                continue
+        try:
+            yield 0
 
-            stderr_line: Union[bytes, None] = await self.process.stdout.readline()
-            if not stderr_line:
-                # Process has finished, check the return code
-                await self.process.wait()
-                if self.process.returncode != 0:
-                    raise RuntimeError(
-                        f"Error running command {self.cmd}: {self.stderr}"
-                    )
-                break
-            stderr_line_str = stderr_line.decode("utf-8", errors="replace").strip()
+            stderr: List[str] = []
+            while True:
+                if self.process.stdout is None:
+                    continue
 
-            progress = self._process_output(stderr_line_str, stderr, duration_override)
-            if progress is not None:
-                yield progress
+                stderr_line: Union[bytes, None] = await self.process.stdout.readline()
+                if not stderr_line:
+                    # Process has finished, check the return code
+                    await self.process.wait()
+                    if self.process.returncode != 0:
+                        raise RuntimeError(
+                            f"Error running command {self.cmd}: {self.stderr}"
+                        )
+                    break
+                stderr_line_str = stderr_line.decode("utf-8", errors="replace").strip()
 
-        yield 100
-        self.process = None
+                progress = self._process_output(
+                    stderr_line_str, stderr, duration_override
+                )
+                if progress is not None:
+                    yield progress
+
+            yield 100
+        except GeneratorExit:
+            # Handle case where async generator is closed prematurely
+            await self._async_cleanup_process()
+            raise
+        except Exception:
+            # Handle any other exception
+            await self._async_cleanup_process()
+            raise
+        finally:
+            # Normal cleanup
+            await self._async_cleanup_process()
+
+    async def _async_cleanup_process(self) -> None:
+        """Clean up the async process."""
+        if self.process is not None:
+            try:
+                if self.process.returncode is None:  # Process is still running
+                    self.process.kill()
+                    try:
+                        await self.process.wait()
+                    except Exception:
+                        pass  # Ignore any errors during cleanup
+            except Exception:
+                pass  # Ignore any errors during cleanup
+            finally:
+                self.process = None
+                # Detach the finalizer since we've cleaned up manually
+                if hasattr(self, "_cleanup_ref"):
+                    self._cleanup_ref.detach()
 
     def quit_gracefully(self) -> None:
         """
